# stdlib imports

# django imports
from django.contrib.auth import get_user_model

# 3rd party imports
from factory import fuzzy, DjangoModelFactory
import factory

# project imports
from djangoflutterwave.models import DRPlanModel, DRTransactionModel


class DRPlanModelFactory(DjangoModelFactory):
    """Factory for the DRPlanModel"""

    name = factory.Faker("word")
    amount = fuzzy.FuzzyDecimal(low=20, high=100, precision=2)
    currency = fuzzy.FuzzyChoice(choices=["USD", "ZAR", "EUR"])
    modal_title = factory.Faker("word")

    class Meta:
        model = DRPlanModel


class UserFactory(DjangoModelFactory):
    """Factory for the User model"""

    username = factory.Faker("word")
    first_name = factory.Faker("first_name")
    last_name = factory.Faker("last_name")
    email = factory.Faker("email")
    password = factory.PostGenerationMethodCall("set_password", "adminadmin")
    is_active = True
    is_staff = False
    is_superuser = False

    class Meta:
        model = get_user_model()


class DRTransactionModelFactory(DjangoModelFactory):
    """Factory for the DRTransactionModel"""

    plan = factory.SubFactory(DRPlanModelFactory)
    user = factory.SubFactory(UserFactory)
    tx_ref = factory.Faker("word")
    flw_ref = factory.Faker("word")
    device_fingerprint = factory.Faker("word")
    amount = fuzzy.FuzzyDecimal(low=20, high=100, precision=2)
    currency = fuzzy.FuzzyChoice(choices=["USD", "ZAR", "EUR"])
    charged_amount = fuzzy.FuzzyDecimal(low=20, high=100, precision=2)
    app_fee = fuzzy.FuzzyDecimal(low=1, high=5, precision=2)
    merchant_fee = fuzzy.FuzzyDecimal(low=1, high=5, precision=2)
    processor_response = factory.Faker("word")
    auth_model = factory.Faker("word")
    ip = factory.Faker("word")
    narration = factory.Faker("word")
    status = fuzzy.FuzzyChoice(choices=["successful", "failed"])
    payment_type = fuzzy.FuzzyChoice(choices=["card", "ussd"])
    created_at = factory.Faker("word")
    account_id = fuzzy.FuzzyInteger(low=1, high=100)

    class Meta:
        model = DRTransactionModel



# stdlib imports

# django imports
from django.contrib.auth.mixins import LoginRequiredMixin
from django.contrib.auth import get_user_model
from django.views.generic import TemplateView

# 3rd party imports
from rest_framework.generics import CreateAPIView
from rest_framework.response import Response
from rest_framework import status

# project imports
from djangoflutterwave.models import DRTransactionModel, DRPlanModel
from djangoflutterwave.serializers import DRTransactionSerializer


UserModel = get_user_model()


class TransactionDetailView(LoginRequiredMixin, TemplateView):
    """Returns a transaction template"""

    template_name = "djangoflutterwave/transaction.html"

    def get_context_data(self, **kwargs):
        """Add transaction to context data"""
        kwargs = super().get_context_data(**kwargs)
        try:
            kwargs["transaction"] = DRTransactionModel.objects.get(
                user=self.request.user, tx_ref=self.kwargs["tx_ref"]
            )
        except DRTransactionModel.DoesNotExist:
            kwargs["transaction"] = None
        return kwargs


class TransactionCreateView(CreateAPIView):
    """Provides an api end point to create transactions"""

    queryset = DRTransactionModel.objects.all()
    serializer_class = DRTransactionSerializer
    authentication_classes: list = []

    def create(self, request, *args, **kwargs):
        """Override create to specify request.data['data'] for serializer data"""
        serializer = self.get_serializer(data=request.data.get("data", None))
        serializer.is_valid(raise_exception=True)
        self.perform_create(serializer)
        headers = self.get_success_headers(serializer.data)
        return Response(
            serializer.data, status=status.HTTP_201_CREATED, headers=headers
        )

    def perform_create(self, serializer: DRTransactionSerializer) -> None:
        """Add plan and user to Transaction instance, determined from the received
        reference"""
        reference = serializer.validated_data["tx_ref"]
        plan_id = reference.split("__")[0]
        user_id = reference.split("__")[2]
        serializer.save(
            user=UserModel.objects.get(id=user_id),
            plan=DRPlanModel.objects.get(id=plan_id),
        )



# Generated by Django 3.1.4 on 2020-12-18 07:48

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='DRPlanModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=50, unique=True)),
                ('amount', models.DecimalField(decimal_places=2, max_digits=9)),
                ('flw_plan_id', models.PositiveIntegerField(blank=True, help_text='Flutterwave plan id. Only required if this is a subscription plan.', null=True, unique=True)),
                ('currency', models.CharField(default='USD', max_length=3)),
                ('modal_logo_url', models.URLField(blank=True, help_text='URL to logo image to be displayed on payment modal.', max_length=500, null=True)),
                ('modal_title', models.CharField(blank=True, help_text='Title to be displayed on payment modal.', max_length=200, null=True)),
                ('pay_button_text', models.CharField(default='Sign Up', help_text='Text used for button when displayed in a template.', max_length=100)),
                ('created_datetime', models.DateTimeField(auto_now_add=True)),
            ],
            options={
                'verbose_name': 'Plan',
                'verbose_name_plural': 'Plans',
            },
        ),
        migrations.CreateModel(
            name='DRTransactionModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_datetime', models.DateTimeField(auto_now_add=True)),
                ('tx_ref', models.CharField(max_length=100)),
                ('flw_ref', models.CharField(max_length=100)),
                ('device_fingerprint', models.CharField(max_length=100)),
                ('amount', models.DecimalField(decimal_places=2, max_digits=9)),
                ('currency', models.CharField(max_length=3)),
                ('charged_amount', models.DecimalField(decimal_places=2, max_digits=9)),
                ('app_fee', models.DecimalField(decimal_places=2, max_digits=9)),
                ('merchant_fee', models.DecimalField(decimal_places=2, max_digits=9)),
                ('processor_response', models.CharField(max_length=100)),
                ('auth_model', models.CharField(max_length=100)),
                ('ip', models.CharField(max_length=100)),
                ('narration', models.CharField(max_length=100)),
                ('status', models.CharField(max_length=50)),
                ('payment_type', models.CharField(max_length=50)),
                ('created_at', models.CharField(max_length=100)),
                ('account_id', models.PositiveIntegerField()),
                ('plan', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='dr_transactions', to='djangoflutterwave.drplanmodel')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='dr_transactions', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'Transaction',
                'verbose_name_plural': 'Transactions',
            },
        ),
    ]



# stdlib imports
from unittest.mock import patch

# django imports
from django.test import TestCase

# 3rd party imports

# project imports
from djangoflutterwave.tests.factories import DRPlanModelFactory, UserFactory
from djangoflutterwave.templatetags.djangoflutterwave_tags import pay_button_params


class TestTemplateTags(TestCase):
    """Test suite for template tags"""

    @patch("djangoflutterwave.templatetags.djangoflutterwave_tags.timezone")
    @patch("djangoflutterwave.templatetags.djangoflutterwave_tags.settings")
    @patch("djangoflutterwave.templatetags.djangoflutterwave_tags.reverse")
    def test_pay_button_params(self, mock_reverse, mock_settings, mock_timezone):
        """Ensure a json string is returned containing the correct tx_ref,
        public_key and redirect_url"""
        mock_reverse.return_value = "test"
        mock_settings.FLW_PUBLIC_KEY = "test"
        mock_timezone.now.return_value.timestamp.return_value = "test"
        plan = DRPlanModelFactory()
        user = UserFactory()

        expected_response = (
            f'{{"tx_ref": "{plan.id}__test__{user.id}"'
            ', "redirect_url": "test", "public_key": "test"}'
        )
        actual_response = pay_button_params(user_pk=user.pk, plan_pk=plan.pk)
        mock_reverse.assert_called()
        self.assertEqual(expected_response, actual_response)



# stdlib imports

# django imports
from django.contrib.auth import get_user_model

# 3rd party imports
from rest_framework import serializers

# project imports
from djangoflutterwave.models import DRTransactionModel, DRPlanModel


UserModel = get_user_model()


class DRTransactionSerializer(serializers.ModelSerializer):
    """Serializer for the DRTransactionModel Model"""

    class Meta:
        model = DRTransactionModel
        fields = (
            "tx_ref",
            "flw_ref",
            "device_fingerprint",
            "amount",
            "currency",
            "charged_amount",
            "app_fee",
            "merchant_fee",
            "processor_response",
            "auth_model",
            "ip",
            "narration",
            "status",
            "payment_type",
            "created_at",
            "account_id",
        )

    def validate_reference(self, value: str) -> str:
        """Ensure the received reference contains a valid plan_id and
        user_id"""
        try:
            plan_id = value.split("__")[0]
            DRPlanModel.objects.get(id=plan_id)
        except DRPlanModel.DoesNotExist:
            raise serializers.ValidationError("Payment type does not exist")

        try:
            user_id = value.split("__")[2]
            UserModel.objects.get(id=user_id)
        except UserModel.DoesNotExist:
            raise serializers.ValidationError("User does not exist")

        return value



# stdlib imports
import json

# django imports
from django.shortcuts import reverse
from django.utils import timezone
from django import template

# 3rd party imports

# project imports
from djangoflutterwave import settings


register = template.Library()


@register.simple_tag()
def pay_button_params(user_pk: str, plan_pk: str) -> str:
    """Returns params required when submitting a payment request to flutterwave.

    Returns:
        tx_ref: created by combining plan_pk, timestamp and user_pk
        redirect_url: transaction detail page to redirect to
        public_key: public key from settings
    """
    now = timezone.now().timestamp()
    tx_ref = f"{plan_pk}__{now}__{user_pk}"
    redirect_url = reverse(
        "djangoflutterwave:transaction_detail", kwargs={"tx_ref": tx_ref}
    )
    return json.dumps(
        {
            "tx_ref": tx_ref,
            "redirect_url": redirect_url,
            "public_key": settings.FLW_PUBLIC_KEY,
        }
    )



# stdlib imports

# django imports
from django.shortcuts import reverse
from django.test import TestCase, RequestFactory

# 3rd party imports
from rest_framework.test import APITestCase, APIRequestFactory

# project imports
from djangoflutterwave.views import TransactionDetailView, TransactionCreateView
from djangoflutterwave.models import DRTransactionModel
from djangoflutterwave.serializers import DRTransactionSerializer
from djangoflutterwave.tests.factories import (
    DRPlanModelFactory,
    UserFactory,
    DRTransactionModelFactory,
)


class TestTransactionDetailView(TestCase):
    """Test suite for the TransactionDetailView"""

    def test_get_context_data(self):
        """Ensure a transaction is added to the context only if a valid user and
        reference is provided"""
        factory = RequestFactory()
        user = UserFactory()
        transaction = DRTransactionModelFactory(user=user)
        request = factory.get("test")
        request.user = user
        view = TransactionDetailView()
        view.request = request

        view.kwargs = {"tx_ref": transaction.tx_ref}
        context_data = view.get_context_data()
        self.assertEqual(transaction, context_data["transaction"])

        view.kwargs = {"tx_ref": "invalid"}
        context_data = view.get_context_data()
        self.assertIsNone(context_data["transaction"])


class TestTransactionCreateView(APITestCase):
    """Test suite for the TransactionCreateView"""

    def test_perform_create(self):
        """Ensure the user and plan are gotten from the reference and saved to the
        Transaction instance"""
        user = UserFactory()
        plan = DRPlanModelFactory()
        factory = APIRequestFactory()
        data = {
            "tx_ref": f"{plan.id}__test__{user.id}",
            "flw_ref": "test",
            "device_fingerprint": "test",
            "amount": 10.00,
            "currency": "USD",
            "charged_amount": 9.00,
            "app_fee": 0.50,
            "merchant_fee": 0.50,
            "processor_response": "test",
            "auth_model": "test",
            "ip": "test",
            "narration": "test",
            "status": "test",
            "payment_type": "test",
            "created_at": "test",
            "account_id": 123,
        }
        request = factory.post("fake-url", data)
        request.user = user
        view = TransactionCreateView()
        view.request = request
        serializer = DRTransactionSerializer(data=data)
        serializer.is_valid()
        view.perform_create(serializer=serializer)

        self.assertEqual(DRTransactionModel.objects.count(), 1)
        transaction = DRTransactionModel.objects.first()
        self.assertEqual(transaction.user.id, user.id)
        self.assertEqual(transaction.plan.id, plan.id)



# stdlib import

# django imports
from django.contrib import admin

# project imports
from djangoflutterwave.models import DRPlanModel, DRTransactionModel

# project imports


class PlanAdmin(admin.ModelAdmin):
    list_display = ("name", "amount", "flw_plan_id")
    search_fields = ("name",)
    readonly_fields = ("created_datetime",)


class TransactionAdmin(admin.ModelAdmin):
    list_display = ("user", "plan", "tx_ref", "amount", "created_datetime")
    search_fields = ("user__username", "plan__name", "tx_ref")
    readonly_fields = ("created_datetime",)


admin.site.register(DRPlanModel, PlanAdmin)
admin.site.register(DRTransactionModel, TransactionAdmin)



# stdlib import

# django imports
from django.conf import settings
from django.contrib.auth import get_user_model
from django.db import models

# 3rd party imports

# project imports

UserModel = get_user_model()


class DRPlanModel(models.Model):
    """Represents either a Plan or OnceOff payment type"""

    name = models.CharField(max_length=50, unique=True)
    amount = models.DecimalField(decimal_places=2, max_digits=9)
    flw_plan_id = models.PositiveIntegerField(
        unique=True,
        blank=True,
        null=True,
        help_text="Flutterwave plan id. Only required if this is a subscription plan.",
    )
    currency = models.CharField(max_length=3, default="USD")
    modal_logo_url = models.URLField(
        max_length=500,
        blank=True,
        null=True,
        help_text="URL to logo image to be displayed on payment modal.",
    )
    modal_title = models.CharField(
        max_length=200,
        blank=True,
        null=True,
        help_text="Title to be displayed on payment modal.",
    )
    pay_button_text = models.CharField(
        max_length=100,
        default="Sign Up",
        help_text="Text used for button when displayed in a template.",
    )
    pay_button_css_classes = models.CharField(
        max_length=200,
        blank=True,
        null=True,
        help_text="css classes to be applied to pay button in template.",
    )
    created_datetime = models.DateTimeField(auto_now_add=True)

    class Meta:
        verbose_name = "Plan"
        verbose_name_plural = "Plans"

    def __str__(self):
        return self.name


class DRTransactionModel(models.Model):
    """Represents a transaction for a specific payment type and user"""

    plan = models.ForeignKey(
        to=DRPlanModel, related_name="dr_transactions", on_delete=models.CASCADE
    )
    user = models.ForeignKey(
        to=UserModel, related_name="dr_transactions", on_delete=models.CASCADE
    )
    created_datetime = models.DateTimeField(auto_now_add=True)
    tx_ref = models.CharField(max_length=100)
    flw_ref = models.CharField(max_length=100)
    device_fingerprint = models.CharField(max_length=100)
    amount = models.DecimalField(decimal_places=2, max_digits=9)
    currency = models.CharField(max_length=3)
    charged_amount = models.DecimalField(decimal_places=2, max_digits=9)
    app_fee = models.DecimalField(decimal_places=2, max_digits=9)
    merchant_fee = models.DecimalField(decimal_places=2, max_digits=9)
    processor_response = models.CharField(max_length=100)
    auth_model = models.CharField(max_length=100)
    ip = models.CharField(max_length=100)
    narration = models.CharField(max_length=100)
    status = models.CharField(max_length=50)
    payment_type = models.CharField(max_length=50)
    created_at = models.CharField(max_length=100)
    account_id = models.PositiveIntegerField()

    class Meta:
        verbose_name = "Transaction"
        verbose_name_plural = "Transactions"

    def __str__(self):
        return self.tx_ref



# Generated by Django 3.1.4 on 2020-12-18 08:48

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djangoflutterwave', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='drplanmodel',
            name='pay_button_css_classes',
            field=models.CharField(blank=True, help_text='css classes to be applied to pay button in template.', max_length=200, null=True),
        ),
    ]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['djangoflutterwave',
 'djangoflutterwave.migrations',
 'djangoflutterwave.templatetags',
 'djangoflutterwave.tests']

package_data = \
{'': ['*'],
 'djangoflutterwave': ['static/djangoflutterwave/js/*',
                       'templates/djangoflutterwave/*']}

install_requires = \
['django>=2.2,<4', 'djangorestframework>=3.10,<4.0', 'requests>=2.22,<3.0']

setup_kwargs = {
    'name': 'djangoflutterwave',
    'version': '0.2.0',
    'description': 'Django integration for Flutterwave payments and subscriptions',
    'long_description': '# Django Flutterwave\n\n## Project Description\n\nThis project provides Django integration for [Flutterwave](https://flutterwave.com/) payments and subscriptions.\n\nCurrent functionality:\n- Allow users to make payments (once off and subscription)\n- Create payment buttons which launch inline payment modals\n- Maintain a payment transaction history linked to users\n\n# Requirements\n\n- Python >= 3.6\n- Django >= 2.0\n\n# Installation\n\n```bash\npip install djangoflutterwave\n```\n\n# Setup\n\nAdd `"djangoflutterwave"` to your `INSTALLED_APPS`\n\nRun Django migrations:\n\n```python\nmanage.py migrate\n```\n\nAdd the following to your `settings.py`:\n\n```python\nFLW_PRODUCTION_PUBLIC_KEY = "your key"\nFLW_PRODUCTION_SECRET_KEY = "your key"\nFLW_SANDBOX_PUBLIC_KEY = "your key"\nFLW_SANDBOX_SECRET_KEY = "your key"\nFLW_SANDBOX = True\n```\n\nThe above config will ensure `djangoflutterwave` uses your sandbox. Once you\'re ready to\ngo live, set `FLW_SANDBOX = False`\n\nAdd `djangoflutterwave` to your `urls.py`:\n\n```python\npath("djangoflutterwave/", include("djangoflutterwave.urls", namespace="djangoflutterwave"))\n```\n\nAdd the following url as a webhook in your Flutterwave dashboard. This will be used by\nFlutterwave to `POST` payment transactions to your site:\n\n```bash\nhttp://yoursite.com/djangoflutterwave/transaction/\n```\n\n`Note:` while in development, a tool like ngrok (or similar) may prove useful to ensure\nyour localhost is accessible to Flutterwave for the above webhook calls.\n\n# Usage\n\n`djangoflutterwave` provides two models, namely:\n\n- The `DRPlanModel` allows you to create `once off` or `subscription` plans. When creating a `subscription` plan, you will need to create the plan in Flutterwave first and then enter the corresonding information as a `DRPlanModel` instance (ie: `flw_plan_id` field corresponds to the Flutterwave `Plan ID`).\n- The `DRTransactionModel` creates transactions when Flutterwave POSTS to the above mentioned webhook url. This provides a history of all transactions (once off or recurring), linked to the relevant `DRPlanModel` and `user`.\n\nA payment button can be created as follows:\n\n1. Create a new plan (ie: `DRPlanModel`) using the django admin.\n2. In the view where you wish the button to appear, add the above created `DRPlanModel` instance to your context, eg:\n\n```python\nfrom djangoflutterwave.models import DRPlanModel\n\nclass SignUpView(TemplateView):\n    """Sign Up view"""\n\n    template_name = "my_payment_template.html"\n\n    def get_context_data(self, **kwargs):\n        """Add payment type to context data"""\n        kwargs = super().get_context_data(**kwargs)\n        kwargs["pro_plan"] = DRPlanModel.objects.filter(\n            name="Pro Plan"\n        ).first()\n        return kwargs\n```\n\n3. In your template, add the button wherever you wish for it to appear as follows:\n\n```python\n{% include \'djangoflutterwave/pay_button.html\' with plan=pro_plan %}\n```\n\n`Note:` You can add multiple buttons to a single template by simply adding multiple\nplans to your context data and then including each of them with their own `include`\ntag as above.\n\n4. Add the following to your django base template (or anywhere in your template heirarchy that ensures it is loaded before your payment buttons):\n\n```html\n<script type="text/javascript" src="https://checkout.flutterwave.com/v3.js"></script>\n<script src="{% static \'djangoflutterwave/js/payment.js\' %}"></script>\n```\n\n# Button Styling\n\nUse the `pay_button_css_classes` field on the `DRPlanModel` model to add css classes to\nbuttons which will be rendered in your template.\n\n# Transaction Detail Page\n\nFollowing a user payment, they will be redirected to the transaction detail page\nlocated at `/djangoflutterwave/<str:tx_ref>/`.\n\nA default transaction detail template is already available, however if you want\nto override it, you may do so by creating a new template in your root\ntemplates directory, ie: `/templates/djangoflutterwave/transaction.html`\n\nYou will have access to `{{ transaction }}` within that template.\n\n# Development and contribution\n\nIf you wish to contribute to the project, there is an example app that demonstrates\ngeneral usage.\n\n### Running the example:\n\n```bash\ngit clone https://github.com/bdelate/django-flutterwave.git\ncd django-flutterwave\n```\n\nCreate file `example/env/dev.env` and populate it with the following:\n\n```bash\nFLW_SANDBOX_PUBLIC_KEY=your_sandbox_public_key\nFLW_SANDBOX_SECRET_KEY=your_sandbox_secret_key\nFLW_PRODUCTION_PUBLIC_KEY=test\nFLW_PRODUCTION_SECRET_KEY=test\n```\n\nRun the following commands:\n\n```bash\nmake build\nmake migrate\nmake import\nmake dup\n```\n\nFlutterwave requires payments to be associated with users who have an email address.\nTherefore, create and login with a new django user or use the existing user which will\nhave been created by the above import command:\n\n```\nusername: admin\npassword: adminadmin\n```\n\nNavigate to http://localhost:8000/',
    'author': 'Brendon Delate',
    'author_email': 'brendon.delate@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/bdelate/django-flutterwave.git',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4',
}


setup(**setup_kwargs)



"""
djangoflutterwave default settings are defined here and determined in combination
with what has been defined in django.conf.settings.
"""

# stdlib import

# django imports
from django.conf import settings

# 3rd party imports

# project imports

FLW_SANDBOX = getattr(settings, "FLW_SANDBOX", True)

if FLW_SANDBOX:
    FLW_PUBLIC_KEY = getattr(settings, "FLW_SANDBOX_PUBLIC_KEY", "not set")
    FLW_SECRET_KEY = getattr(settings, "FLW_SANDBOX_SECRET_KEY", "not set")
else:
    FLW_PUBLIC_KEY = getattr(settings, "FLW_PRODUCTION_PUBLIC_KEY", "not set")
    FLW_SECRET_KEY = getattr(settings, "FLW_PRODUCTION_SECRET_KEY", "not set")
